@@ -8,7 +8,8 @@ use aftershock::matrix3::*;
 use aftershock::math::*;
 use aftershock::color::*;
 use aftershock::drawables::*;
-use aftershock::random::*;
+
+use squares_rng::SquaresRNG;
 
 use std::time::Instant;
 
@@ -39,8 +40,8 @@ pub struct Player {
     pub active: bool,
     pub velocity: Vector2,
     pub position: Vector2,
-    pub rotation: f32,
-    pub radius: f32,
+    pub rotation: f64,
+    pub radius: f64,
     pub scale: Vector2,
 }
 
@@ -49,10 +50,10 @@ pub struct Bullet {
     pub active: bool,
     pub velocity: Vector2,
     pub position: Vector2,
-    pub rotation: f32,
-    pub radius: f32,
+    pub rotation: f64,
+    pub radius: f64,
     pub scale: Vector2,
-    pub lifetime: f32,
+    pub lifetime: f64,
 }
 
 impl Bullet {
@@ -75,8 +76,8 @@ pub struct Asteroid {
     pub shape: [Vector2; 8],
     pub velocity: Vector2,
     pub position: Vector2,
-    pub rotation: f32,
-    pub radius: f32,
+    pub rotation: f64,
+    pub radius: f64,
     pub scale: Vector2,
     pub health: u8,
 }
@@ -95,7 +96,7 @@ impl Asteroid {
         }
     }
 
-    pub fn generate_shape(radius: f32, rng: &mut Random) -> [Vector2; 8] {
+    pub fn generate_shape(radius: f64, rng: &mut SquaresRNG) -> [Vector2; 8] {
         let mut points: [Vector2; 8] = [
             Vector2::new(1.0, 0.0), // Right
             Vector2::new(0.5, 0.5), // Bottom Right
@@ -111,7 +112,7 @@ impl Asteroid {
         // Does not effect collisions
         for p in points.iter_mut() {
             *p *= radius;
-            *p += Vector2::new(rng.randf_range(-2.0, 2.0), rng.randf_range(-2.0, 2.0));
+            *p += Vector2::new(rng.rangef64(-2.0, 2.0), rng.rangef64(-2.0, 2.0));
         }
 
         points
@@ -122,7 +123,7 @@ impl Asteroid {
 pub struct ExplosionParticle {
     pub position: Vector2,
     pub velocity: Vector2,
-    pub radius: f32,
+    pub radius: f64,
 }
 
 impl ExplosionParticle {
@@ -136,7 +137,7 @@ impl ExplosionParticle {
 }
 
 /// If the squared distance between the two points is smaller than the combined diameter's squared, then the two circles are overlapping!
-pub fn circle_overlap(p1: Vector2, r1: f32, p2: Vector2, r2: f32) -> bool {
+pub fn circle_overlap(p1: Vector2, r1: f64, p2: Vector2, r2: f64) -> bool {
     // Double the incoming radius's so they are diameter's instead.
     let (r1, r2) = (r1 * 2.0, r2 * 2.0);
     (p2.x - p1.x) * (p2.x - p1.x) + (p2.y - p1.y) * (p2.y - p1.y) < (r1*r2) + (r1*r2)
@@ -144,7 +145,7 @@ pub fn circle_overlap(p1: Vector2, r1: f32, p2: Vector2, r2: f32) -> bool {
 
 pub struct AsteroidsEngine {
     pub camera: Matrix3,
-    pub camera_boomzoom: f32,
+    pub camera_boomzoom: f64,
 
     pub player: Player,
     pub asteroids: [Asteroid; 128],
@@ -152,7 +153,7 @@ pub struct AsteroidsEngine {
 
     pub explosion_particles: [ExplosionParticle; 8192],
 
-    pub score: i32,
+    pub score: i64,
 
     pub uidx_asteroids: usize,
     pub uidx_bullets: usize,
@@ -166,21 +167,21 @@ pub struct AsteroidsEngine {
 
     pub paused: bool,
 
-    pub rng: Random,
-    pub rng_number: f32,
+    pub rng: SquaresRNG,
+    pub rng_number: f64,
 
     pub sf_score: SpriteFont,
 
     pub debug_collision: bool,
     pub debug_info: bool,
 
-    pub realtime: f32,
-    pub timescale: f32,
+    pub realtime: f64,
+    pub timescale: f64,
     pub tics: u64,
     pub fps: u64,
     pub fps_print: u64,
-    pub dt: f32,
-    pub dt_unscaled: f32,
+    pub dt: f64,
+    pub dt_unscaled: f64,
 
     dt_before: Instant,
 }
@@ -192,12 +193,14 @@ impl AsteroidsEngine {
         let rng_seedcounter = std::time::SystemTime::now().duration_since(std::time::UNIX_EPOCH).expect("DONT. FUCK. WITH TIME.").as_secs();
         let font_glyphidx = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!?*^&()[]<>-+=/\\\"'`~:;,.%abcdefghijklmnopqrstuvwxyz";
 
+        let rng_key: u64 = 0xd2f6ae576fced215; // Key requires even distribution of 0's and 1's
+
         AsteroidsEngine {
             camera: Matrix3::identity(),
             camera_boomzoom: 1.0,
 
             // Use time in seconds as counter seed, and use the first RNG key in the table.
-            rng: Random::new(rng_seedcounter, 0),
+            rng: SquaresRNG::new_with_key(rng_seedcounter, rng_key), 
             rng_number: 0.0,
 
             player: Player { active: true, velocity: Vector2::new(0.0, 0.0), position: Vector2::new(256.0, 256.0), rotation: 0.0, radius: 4.0, scale: Vector2::one()},
@@ -298,7 +301,7 @@ impl AsteroidsEngine {
 		let mut sys_spritefont: SpriteFont = SpriteFont::new("core/tiny_font.png", font_glyphidx, 5, 5, 7.0, 14.0);
     
 
-        let mut printtime: f32 = 0.0;
+        let mut printtime: f64 = 0.0;
 
         'running: loop {
             self.update_times();
@@ -317,7 +320,7 @@ impl AsteroidsEngine {
                 self.fps_print = self.fps;
                 self.fps = 0;
                 printtime = 0.0;
-                self.rng_number = self.rng.randf_linear01();
+                self.rng_number = self.rng.randf64();
             }
 
             self.update_controls(&event_pump);
@@ -355,7 +358,7 @@ impl AsteroidsEngine {
             // We HAVE to do this in case the window is resized, otherwise the screen texture would override anything in the window anyways
             canvas.clear();
 
-            self.rasterizer.cls();
+            self.rasterizer.clear();
 
 
 
@@ -482,8 +485,8 @@ impl AsteroidsEngine {
             }
     
             self.player.position += self.player.velocity * self.dt;
-            self.player.position.x = modf(self.player.position.x, RENDER_WIDTH as f32);
-            self.player.position.y = modf(self.player.position.y, RENDER_HEIGHT as f32);
+            self.player.position.x = modf(self.player.position.x, RENDER_WIDTH as f64);
+            self.player.position.y = modf(self.player.position.y, RENDER_HEIGHT as f64);
 
             // Check if we are touching any asteroids or bullets. Escape if player is already dead
             for i in 0..self.asteroids.len() {
@@ -546,10 +549,10 @@ impl AsteroidsEngine {
         };
 
         //self.rasterizer.wrapping = true;
-        self.rasterizer.pline(mtx_line0.x as i32, mtx_line0.y as i32, mtx_line1.x as i32, mtx_line1.y as i32, player_color);
-        self.rasterizer.pline(mtx_line1.x as i32, mtx_line1.y as i32, mtx_line2.x as i32, mtx_line2.y as i32, player_color);
-        self.rasterizer.pline(mtx_line2.x as i32, mtx_line2.y as i32, mtx_line3.x as i32, mtx_line3.y as i32, player_color);
-        self.rasterizer.pline(mtx_line3.x as i32, mtx_line3.y as i32, mtx_line0.x as i32, mtx_line0.y as i32, player_color);
+        self.rasterizer.pline(mtx_line0.x as i64, mtx_line0.y as i64, mtx_line1.x as i64, mtx_line1.y as i64, player_color);
+        self.rasterizer.pline(mtx_line1.x as i64, mtx_line1.y as i64, mtx_line2.x as i64, mtx_line2.y as i64, player_color);
+        self.rasterizer.pline(mtx_line2.x as i64, mtx_line2.y as i64, mtx_line3.x as i64, mtx_line3.y as i64, player_color);
+        self.rasterizer.pline(mtx_line3.x as i64, mtx_line3.y as i64, mtx_line0.x as i64, mtx_line0.y as i64, player_color);
         //self.rasterizer.wrapping = false;
     }
 
@@ -562,12 +565,12 @@ impl AsteroidsEngine {
         for i in 0..self.explosion_particles.len() {
             if spawn_counter > 0 {
                 if self.explosion_particles[i].radius <= 0.1 {
-                    self.explosion_particles[i].radius = self.rng.randf_range(4.0, 16.0);
+                    self.explosion_particles[i].radius = self.rng.rangef64(4.0, 16.0);
                     self.explosion_particles[i].position = position;
                     self.explosion_particles[i].velocity = Vector2::new(
-                        self.rng.randf_range(-1.0, 1.0),
-                        self.rng.randf_range(-1.0, 1.0)
-                    ) * self.rng.randf_range(8.0, 256.0);
+                        self.rng.rangef64(-1.0, 1.0),
+                        self.rng.rangef64(-1.0, 1.0)
+                    ) * self.rng.rangef64(8.0, 256.0);
                     spawn_counter -= 1;
                 }
             } else {
@@ -592,9 +595,9 @@ impl AsteroidsEngine {
             if self.explosion_particles[i].radius > 0.1 {
                 let mtx_position = self.camera.forward(self.explosion_particles[i].position);
                 self.rasterizer.pcircle(false, 
-                    mtx_position.x as i32,
-                    mtx_position.y as i32,
-                    (self.explosion_particles[i].radius * self.camera_boomzoom) as i32, Color::hsv(0.1, 1.0, self.explosion_particles[i].radius / 8.0)
+                    mtx_position.x as i64,
+                    mtx_position.y as i64,
+                    (self.explosion_particles[i].radius * self.camera_boomzoom) as i64, Color::hsv(0.1, 1.0, self.explosion_particles[i].radius / 8.0)
                 );
             }
         }
@@ -610,8 +613,8 @@ impl AsteroidsEngine {
         // We need to move the camera closer to the center based on zoom since it's technically in the top-left corner
         let camera_translated = Matrix3::translated(
             Vector2::new(
-                lerpf(RENDER_WIDTH as f32 / 2.0, 0.0, self.camera_boomzoom), 
-                lerpf(RENDER_HEIGHT as f32 / 2.0, 0.0, self.camera_boomzoom)
+                lerpf(RENDER_WIDTH as f64 / 2.0, 0.0, self.camera_boomzoom), 
+                lerpf(RENDER_HEIGHT as f64 / 2.0, 0.0, self.camera_boomzoom)
             ));
 
         self.camera = camera_scaled * camera_translated;
@@ -628,7 +631,7 @@ impl AsteroidsEngine {
 
     ///// ====== BULLET ====== /////
 
-    pub fn spawn_bullet(&mut self, offset: Vector2, direction: Vector2, force: f32) {
+    pub fn spawn_bullet(&mut self, offset: Vector2, direction: Vector2, force: f64) {
         let mut bullet = &mut self.bullets[self.uidx_bullets % self.bullets.len()];
 
         bullet.position = offset;
@@ -648,8 +651,8 @@ impl AsteroidsEngine {
         for i in 0..self.bullets.len() {
             if self.bullets[i].active {
                 self.bullets[i].position += self.bullets[i].velocity * self.dt;
-                self.bullets[i].position.x = modf(self.bullets[i].position.x, RENDER_WIDTH as f32);
-                self.bullets[i].position.y = modf(self.bullets[i].position.y, RENDER_WIDTH as f32);
+                self.bullets[i].position.x = modf(self.bullets[i].position.x, RENDER_WIDTH as f64);
+                self.bullets[i].position.y = modf(self.bullets[i].position.y, RENDER_WIDTH as f64);
                 self.bullets[i].lifetime += self.dt;
                 if self.bullets[i].lifetime > 5.0 {
                     self.bullets[i].active = false;
@@ -662,7 +665,7 @@ impl AsteroidsEngine {
         for bullet in &self.bullets {
             if bullet.active {
                 let mtx_position = self.camera.forward(bullet.position);
-                self.rasterizer.pcircle(true, mtx_position.x as i32, mtx_position.y as i32, bullet.radius as i32, Color::white());
+                self.rasterizer.pcircle(true, mtx_position.x as i64, mtx_position.y as i64, bullet.radius as i64, Color::white());
             }   
         }
     }
@@ -677,11 +680,11 @@ impl AsteroidsEngine {
     pub fn spawn_asteroid(&mut self) {
         let mut asteroid = &mut self.asteroids[self.uidx_asteroids % self.asteroids.len()];
 
-        asteroid.radius = self.rng.randf_range(4.0, 16.0);
+        asteroid.radius = self.rng.rangef64(4.0, 16.0);
         asteroid.shape = Asteroid::generate_shape(asteroid.radius, &mut self.rng);
-        asteroid.position = Vector2::new(self.rng.randf_range(0.0, RENDER_WIDTH as f32), self.rng.randf_range(0.0, RENDER_HEIGHT as f32));
-        asteroid.rotation = self.rng.randf_range(0.0, 6.28);
-        asteroid.velocity = Vector2::new(self.rng.randf_range(-1.0, 1.0), self.rng.randf_range(-1.0, 1.0)) * self.rng.randf_range(2.0, 64.0);
+        asteroid.position = Vector2::new(self.rng.rangef64(0.0, RENDER_WIDTH as f64), self.rng.rangef64(0.0, RENDER_HEIGHT as f64));
+        asteroid.rotation = self.rng.rangef64(0.0, 6.28);
+        asteroid.velocity = Vector2::new(self.rng.rangef64(-1.0, 1.0), self.rng.rangef64(-1.0, 1.0)) * self.rng.rangef64(2.0, 64.0);
         asteroid.active = true;
 
         self.uidx_asteroids += 1;
@@ -696,8 +699,8 @@ impl AsteroidsEngine {
                 asteroids_active += 1;
 
                 self.asteroids[i].position += self.asteroids[i].velocity * self.dt;
-                self.asteroids[i].position.x = modf(self.asteroids[i].position.x, RENDER_WIDTH as f32);
-                self.asteroids[i].position.y = modf(self.asteroids[i].position.y, RENDER_HEIGHT as f32);
+                self.asteroids[i].position.x = modf(self.asteroids[i].position.x, RENDER_WIDTH as f64);
+                self.asteroids[i].position.y = modf(self.asteroids[i].position.y, RENDER_HEIGHT as f64);
 
                 
 
@@ -748,14 +751,14 @@ impl AsteroidsEngine {
 
                 // Draw lines to rasterizer with wrapping
                 //self.rasterizer.wrapping = true;
-                self.rasterizer.pline(mtx_line0.x as i32, mtx_line0.y as i32, mtx_line1.x as i32, mtx_line1.y as i32, Color::white());
-                self.rasterizer.pline(mtx_line1.x as i32, mtx_line1.y as i32, mtx_line2.x as i32, mtx_line2.y as i32, Color::white());
-                self.rasterizer.pline(mtx_line2.x as i32, mtx_line2.y as i32, mtx_line3.x as i32, mtx_line3.y as i32, Color::white());
-                self.rasterizer.pline(mtx_line3.x as i32, mtx_line3.y as i32, mtx_line4.x as i32, mtx_line4.y as i32, Color::white());
-                self.rasterizer.pline(mtx_line4.x as i32, mtx_line4.y as i32, mtx_line5.x as i32, mtx_line5.y as i32, Color::white());
-                self.rasterizer.pline(mtx_line5.x as i32, mtx_line5.y as i32, mtx_line6.x as i32, mtx_line6.y as i32, Color::white());
-                self.rasterizer.pline(mtx_line6.x as i32, mtx_line6.y as i32, mtx_line7.x as i32, mtx_line7.y as i32, Color::white());
-                self.rasterizer.pline(mtx_line7.x as i32, mtx_line7.y as i32, mtx_line0.x as i32, mtx_line0.y as i32, Color::white());
+                self.rasterizer.pline(mtx_line0.x as i64, mtx_line0.y as i64, mtx_line1.x as i64, mtx_line1.y as i64, Color::white());
+                self.rasterizer.pline(mtx_line1.x as i64, mtx_line1.y as i64, mtx_line2.x as i64, mtx_line2.y as i64, Color::white());
+                self.rasterizer.pline(mtx_line2.x as i64, mtx_line2.y as i64, mtx_line3.x as i64, mtx_line3.y as i64, Color::white());
+                self.rasterizer.pline(mtx_line3.x as i64, mtx_line3.y as i64, mtx_line4.x as i64, mtx_line4.y as i64, Color::white());
+                self.rasterizer.pline(mtx_line4.x as i64, mtx_line4.y as i64, mtx_line5.x as i64, mtx_line5.y as i64, Color::white());
+                self.rasterizer.pline(mtx_line5.x as i64, mtx_line5.y as i64, mtx_line6.x as i64, mtx_line6.y as i64, Color::white());
+                self.rasterizer.pline(mtx_line6.x as i64, mtx_line6.y as i64, mtx_line7.x as i64, mtx_line7.y as i64, Color::white());
+                self.rasterizer.pline(mtx_line7.x as i64, mtx_line7.y as i64, mtx_line0.x as i64, mtx_line0.y as i64, Color::white());
                 //self.rasterizer.wrapping = false;
             }
             
@@ -769,12 +772,12 @@ impl AsteroidsEngine {
 
     ///// ====== ENGINE ====== /////
 
-    pub fn add_score(&mut self, score: i32) {
+    pub fn add_score(&mut self, score: i64) {
         self.score += score;
         self.sf_score.opacity = 255;
     }
 
-    pub fn remove_score(&mut self, score: i32) {
+    pub fn remove_score(&mut self, score: i64) {
         self.score -= score;
         if self.score < 0 {
             self.score = 0;
@@ -784,41 +787,42 @@ impl AsteroidsEngine {
     pub fn draw_score(&mut self) {
         self.sf_score.text = format!("{:0>8}", self.score);
         self.sf_score.scale = Vector2::new(10.0, 10.0);
-        self.sf_score.spacing_horizontal = 64.0;
+        self.sf_score.spacing_horizontal = 6.0;
         self.sf_score.opacity = if self.sf_score.opacity > 32 { self.sf_score.opacity - 1} else { 32 };
-        self.sf_score.position = Vector2::new(8.0, 450.0);
+        self.sf_score.position = Vector2::new(8.0, 450.0) / 10.0;
         self.sf_score.draw(&mut self.rasterizer);
     }
 
     pub fn draw_performance_text(&mut self, spritefont: &mut SpriteFont) {
-        let total_pixels = self.rasterizer.drawn_pixels_since_cls;
+        let total_pixels = self.rasterizer.drawn_pixels_since_clear;
+        spritefont.spacing_horizontal = 4.0;
         spritefont.text = format!("{:.1}ms  ({} UPS) pxd: {}\ncontrols: {}\n{}", (self.dt_unscaled * 100000.0).ceil() / 100.0, self.fps_print, total_pixels, self.controls, self.rng_number);
-        spritefont.scale = Vector2::new(2.0, 2.0);
+        spritefont.scale = Vector2::new(1.0, 1.0);
         spritefont.position = Vector2::new(8.0, 8.0);
         spritefont.opacity = 128;
         spritefont.draw(&mut self.rasterizer);
     }
 
     pub fn draw_debug_collision(&mut self) {
-        self.rasterizer.pcircle(false, self.player.position.x as i32, self.player.position.y as i32, self.player.radius as i32, Color::green());
+        self.rasterizer.pcircle(false, self.player.position.x as i64, self.player.position.y as i64, self.player.radius as i64, Color::green());
 
         for asteroid in &self.asteroids {
             if asteroid.active {
-                self.rasterizer.pcircle(false, asteroid.position.x as i32, asteroid.position.y as i32, asteroid.radius as i32, Color::green());
+                self.rasterizer.pcircle(false, asteroid.position.x as i64, asteroid.position.y as i64, asteroid.radius as i64, Color::green());
             }
         }
 
         for bullet in &self.bullets {
             if bullet.active {
-                self.rasterizer.pcircle(false, bullet.position.x as i32, bullet.position.y as i32, bullet.radius as i32, Color::green());
+                self.rasterizer.pcircle(false, bullet.position.x as i64, bullet.position.y as i64, bullet.radius as i64, Color::green());
             }
         }
     }
 
     pub fn update_times(&mut self) {
         let now = Instant::now();
-        let now_s = (now.elapsed().as_secs() as f32) + (now.elapsed().subsec_nanos() as f32 * 1.0e-9);
-        let before_s = (self.dt_before.elapsed().as_secs() as f32) + (self.dt_before.elapsed().subsec_nanos() as f32 * 1.0e-9);
+        let now_s = (now.elapsed().as_secs() as f64) + (now.elapsed().subsec_nanos() as f64 * 1.0e-9);
+        let before_s = (self.dt_before.elapsed().as_secs() as f64) + (self.dt_before.elapsed().subsec_nanos() as f64 * 1.0e-9);
         self.dt_unscaled = before_s - now_s;
         
         self.dt_before = Instant::now();
@@ -831,7 +835,7 @@ impl AsteroidsEngine {
 
     pub fn restart_game(&mut self) {
         self.player.active = true;
-        self.player.position = Vector2::new(RENDER_WIDTH as f32 / 2.0, RENDER_HEIGHT as f32 / 2.0);
+        self.player.position = Vector2::new(RENDER_WIDTH as f64 / 2.0, RENDER_HEIGHT as f64 / 2.0);
         self.player.velocity = Vector2::zero();
         self.player.rotation = 0.0;
 
